from typing import Any, List, Literal, Optional, Union
from uuid import UUID

import requests
from multimethod import multimethod

from grai_client.endpoints.utilities import response_status_check
from grai_client.endpoints.v1.client import ClientV1
from grai_client.schemas.edge import EdgeLabels, EdgeV1
from grai_client.schemas.node import NodeID, NodeLabels, NodeV1


@ClientV1.get_url.register
def get_node_url(client: ClientV1, obj: NodeID) -> str:
    base_url = client.node_endpoint
    return base_url


@ClientV1.get_url.register
def get_node_url(client: ClientV1, obj: NodeV1) -> str:
    return client.node_endpoint


@ClientV1.get_url.register
def get_edge_url(client: ClientV1, obj: EdgeV1) -> str:
    return client.edge_endpoint


@ClientV1.get_url.register
def get_edge_url(client: ClientV1, obj: NodeLabels) -> str:
    return client.node_endpoint


@ClientV1.get_url.register
def get_edge_url(client: ClientV1, obj: EdgeLabels) -> str:
    return client.edge_endpoint



import abc
import json
from typing import Any, Dict, List, Optional, Sequence, Union

import requests
from multimethod import multimethod

from grai_client.authentication import APIKeyHeader, UserNameHeader, UserTokenHeader
from grai_client.endpoints.utilities import GraiEncoder, response_status_check
from grai_client.schemas.schema import GraiType


class BaseClient(abc.ABC):
    id = "base"

    def __init__(self, host: str, port: str):
        self.host = host
        self.port = port

        prefix = "https" if self.port == "443" else "http"
        self.url = f"{prefix}://{self.host}:{self.port}"
        self.api = f"{self.url}"
        self._auth_headers = None

        if not self.check_server_status():
            raise Exception(f"Server at {self.url} is not responding.")

    def check_server_status(self):
        resp = requests.get(f"{self.url}/health/")
        return resp.status_code == 200

    def build_url(self, endpoint: str) -> str:
        return f"{self.api}{endpoint}"

    @property
    def auth_headers(self) -> Dict:
        if not self._auth_headers:
            raise Exception(
                "Client not authenticated. Please call `set_authentication_headers` with your credentials first"
            )
        return self._auth_headers

    def set_authentication_headers(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
        token: Optional[str] = None,
        api_key: Optional[str] = None,
    ):
        if username and password:
            self._auth_headers = UserNameHeader(username, password, self.url).headers
        elif token:
            self._auth_headers = UserTokenHeader(token).headers
        elif api_key:
            self._auth_headers = APIKeyHeader(api_key).headers
        else:
            raise Exception(
                "Authentication requires either a user token, api key, or username/password combo."
            )

    def check_authentication(self):
        raise NotImplementedError(f"No authentication implemented for {type(self)}")

    @multimethod
    def get_url(self, grai_type: Any) -> str:
        raise NotImplementedError(
            f"No url method implemented for type {type(grai_type)}"
        )

    @multimethod
    def get(self, grai_type: Any) -> Dict:
        raise NotImplementedError(
            f"No get method implemented for type {type(grai_type)}"
        )

    @multimethod
    def post(self, grai_type: Any) -> Dict:
        raise NotImplementedError(
            f"No post method implemented for type {type(grai_type)}"
        )

    @multimethod
    def patch(self, grai_type: Any) -> Dict:
        raise NotImplementedError(
            f"No patch method implemented for type {type(grai_type)}"
        )

    @multimethod
    def delete(self, grai_type: Any) -> Dict:
        raise NotImplementedError(
            f"No delete method implemented for type {type(grai_type)}"
        )


@BaseClient.post.register
def post_sequence(client: BaseClient, objs: Sequence) -> List[Dict]:
    result = [client.post(obj) for obj in objs]
    return result


@BaseClient.patch.register
def patch_sequence(client: BaseClient, objs: Sequence) -> List[Dict]:
    result = [client.patch(obj) for obj in objs]
    return result


@BaseClient.delete.register
def delete_sequence(client: BaseClient, objs: Sequence[GraiType]):
    for obj in objs:
        client.delete(obj)


@BaseClient.get.register
def get_sequence(client: BaseClient, objs: Sequence) -> List[Dict]:
    result = [client.get(obj) for obj in objs]
    return result


@BaseClient.get.register
def get_url_v1(client: BaseClient, url: str) -> requests.Response:
    response = requests.get(url, headers=client.auth_headers)
    response_status_check(response)
    return response


@BaseClient.delete.register
def delete_url_v1(client: BaseClient, url: str) -> requests.Response:
    response = requests.delete(url, headers=client.auth_headers)
    response_status_check(response)
    return response


@BaseClient.patch.register
def patch_url_v1(client: BaseClient, url: str, payload: Dict) -> requests.Response:
    headers = {**client.auth_headers, "Content-Type": "application/json"}
    payload = {k: v for k, v in payload.items() if v is not None}
    response = requests.patch(
        url, data=json.dumps(payload, cls=GraiEncoder), headers=headers
    )

    response_status_check(response)
    return response


@BaseClient.post.register
def post_url_v1(client: BaseClient, url: str, payload: Dict) -> requests.Response:
    headers = {**client.auth_headers, "Content-Type": "application/json"}
    payload = {k: v for k, v in payload.items() if v is not None}
    response = requests.post(
        url, data=json.dumps(payload, cls=GraiEncoder), headers=headers
    )
    response_status_check(response)
    return response



from typing import Dict, Literal, Optional, Union
from uuid import UUID

from pydantic import BaseModel, Field
from typing_extensions import Annotated

from grai_client.schemas.utilities import BaseSpec, GraiBaseModel, PlaceHolderSchema


class BaseNode(GraiBaseModel):
    type: Literal["Node"] = "Node"

    def __hash__(self):
        return


class NodeID(BaseModel):
    name: str
    namespace: str
    id: Optional[UUID]

    def __hash__(self):
        return hash(hash(self.name) + hash(self.namespace))


class V1(NodeID, BaseSpec):
    data_source: str
    display_name: Optional[str]
    is_active: Optional[bool] = True
    metadata: Optional[Dict] = {}


class V2(PlaceHolderSchema, V1):
    pass


class NodeV1(BaseNode):
    version: Literal["v1"]
    spec: V1

    @classmethod
    def from_spec(cls, spec_dict: Dict) -> "NodeV1":
        args = {
            "version": "v1",
            "type": "Node",
            "spec": spec_dict,
        }
        return cls(**args)


class NodeV2(BaseNode):
    version: Literal["v2"]
    spec: V2

    def from_spec(self, spec_dict: Dict) -> "NodeV2":
        raise NotImplementedError()


NodeLabels = Literal["nodes", "node", "Node", "Nodes"]
NodeTypes = Union[NodeV1, NodeV2]
Node = Annotated[NodeTypes, Field(discriminator="version")]



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List, Optional, TypeVar, Union

from pydantic import BaseModel, root_validator

if TYPE_CHECKING:
    from grai_client.schemas.edge import Edge
    from grai_client.schemas.node import Node

    T = TypeVar("T", Node, Edge)


def unpack_object(obj: Union[Dict, BaseModel]) -> Dict:
    if isinstance(obj, Dict):
        return obj
    elif isinstance(obj, BaseModel):
        return obj.dict()
    else:
        raise NotImplementedError(f"No method to unpack objects of type {type(obj)}")


def merge_dicts(a: Dict, b: Dict) -> Dict:
    for k, v in b.items():
        if isinstance(a.get(k, None), dict) and isinstance(v, dict):
            merge_dicts(a[k], v)
        else:
            a[k] = v
    return a


def merge_models(a: T, b: T) -> T:
    a_type = type(a)
    merged = merge_dicts(unpack_object(a), unpack_object(b))
    return a_type(**merged)


class GraiBaseModel(BaseModel):
    # class Config:
    #     frozen = True

    def update(self, new_values: Dict) -> BaseModel:
        values = self.dict()
        return type(self)(**merge_dicts(values, new_values))


class BaseSpec(GraiBaseModel):
    is_active: Optional[bool] = True


class PlaceHolderSchema(BaseSpec):
    @root_validator(pre=True)
    def _(cls, values):
        message = (
            "Something is wrong... I can feel it 😡. You've reached a placeholder schema - "
            "most likely the `version` of your config file doesn't exist yet."
        )
        raise AssertionError(message)



import uuid

from grai_client.schemas.edge import EdgeV1
from grai_client.schemas.node import NodeV1
from grai_client.schemas.schema import Schema


def mock_v1_node(
    name=None,
    namespace=None,
    data_source=None,
    display_name=None,
    is_active=True,
    metadata={},
):
    node_dict = {
        "type": "Node",
        "version": "v1",
        "spec": {
            "id": None,
            "name": str(uuid.uuid4()) if name is None else name,
            "namespace": "client_mocking" if namespace is None else namespace,
            "data_source": str(uuid.uuid4()) if data_source is None else data_source,
            "display_name": str(uuid.uuid4()) if display_name is None else display_name,
            "is_active": is_active,
            "metadata": metadata,
        },
    }
    return NodeV1(**node_dict)


def mock_node_id():
    return {"name": str(uuid.uuid4()), "namespace": str(uuid.uuid4())}


def mock_v1_edge(
    name=None,
    namespace=None,
    data_source=None,
    source=None,
    destination=None,
    is_active=True,
    metadata={},
):
    edge_dict = {
        "type": "Edge",
        "version": "v1",
        "spec": {
            "id": None,
            "name": str(uuid.uuid4()) if name is None else name,
            "namespace": str(uuid.uuid4()) if name is None else name,
            "data_source": str(uuid.uuid4()) if data_source is None else data_source,
            "source": mock_node_id() if source is None else source,
            "destination": mock_node_id() if destination is None else destination,
            "is_active": is_active,
            "metadata": metadata,
        },
    }
    return EdgeV1(**edge_dict)


def mock_v1_edge_and_nodes(
    name=None, data_source=None, is_active=True, metadata={}, namespace=None
):
    node1 = mock_v1_node(namespace=namespace, data_source=data_source)
    node2 = mock_v1_node(namespace=namespace, data_source=data_source)

    edge_dict = {
        "type": "Edge",
        "version": "v1",
        "spec": {
            "id": None,
            "name": str(uuid.uuid4()) if name is None else name,
            "namespace": str(uuid.uuid4()) if name is None else name,
            "data_source": str(uuid.uuid4()) if data_source is None else data_source,
            "source": {k: getattr(node1.spec, k) for k in ["name", "namespace"]},
            "destination": {k: getattr(node2.spec, k) for k in ["name", "namespace"]},
            "is_active": is_active,
            "metadata": metadata,
        },
    }
    return EdgeV1(**edge_dict), [node1, node2]



from typing import List, Sequence, Type

from grai_client.endpoints import client, utilities, v1


def list_clients() -> Sequence[Type[client.BaseClient]]:
    clients = [v1.client.ClientV1]
    return clients



from grai_client.endpoints.v1.client import ClientV1
from grai_client.schemas.edge import EdgeV1
from grai_client.schemas.node import NodeID, NodeV1


@ClientV1.post.register
def post_node_v1(client: ClientV1, grai_type: NodeV1) -> NodeV1:
    url = client.node_endpoint
    response = client.post(url, grai_type.spec.dict())
    return NodeV1.from_spec(response.json())


def process_node_id(client: ClientV1, grai_type: NodeID) -> NodeID:
    """
    Process a NodeID object, either by returning if it has a known id, or by getting
    the id from the server.
    """
    if grai_type.id is not None:
        return grai_type

    server_node = client.get(grai_type)
    if server_node is None:
        message = (
            f"Could not find node with namespace=`{grai_type.namespace} "
            f"and name=`{grai_type.name}` on server"
        )
        raise ValueError(message)

    return server_node


@ClientV1.post.register
def post_edge_v1(client: ClientV1, grai_type: EdgeV1) -> EdgeV1:
    url = client.edge_endpoint

    source = process_node_id(client, grai_type.spec.source)
    destination = process_node_id(client, grai_type.spec.destination)

    payload = grai_type.spec.dict()
    payload["source"] = source.id
    payload["destination"] = destination.id
    response = client.post(url, payload).json()

    if response is not None:
        response["source"] = source
        response["destination"] = destination
    return EdgeV1.from_spec(response)



from itertools import chain
from typing import Any, List, Literal, Optional, TypeVar, Union
from uuid import UUID

import requests
from multimethod import multimethod

from grai_client.endpoints.utilities import response_status_check
from grai_client.endpoints.v1.client import ClientV1
from grai_client.schemas.edge import EdgeLabels, EdgeV1, NodeID
from grai_client.schemas.node import NodeLabels, NodeV1

T = TypeVar("T", NodeV1, EdgeV1)


@ClientV1.get.register
def get_node_id(client: ClientV1, grai_type: NodeID) -> Optional[NodeID]:
    base_url = client.node_endpoint
    if grai_type.id:
        return grai_type

    url = f"{base_url}?name={grai_type.name}&namespace={grai_type.namespace}"
    resp = client.get(url).json()
    if len(resp) == 0:
        return None
    spec = NodeV1.from_spec(resp[0]).spec
    return NodeID(id=spec.id, name=spec.name, namespace=spec.namespace)


@ClientV1.get.register
def get_node_v1(client: ClientV1, grai_type: NodeV1) -> Optional[NodeV1]:
    base_url = client.node_endpoint
    if grai_type.spec.id is not None:
        url = f"{base_url}{grai_type.spec.id}"
        resp = client.get(url).json()
    else:
        url = f"{base_url}?name={grai_type.spec.name}&namespace={grai_type.spec.namespace}"
        resp = client.get(url).json()
        if len(resp) == 0:
            return None
        resp = resp[0]
    return NodeV1.from_spec(resp)


@ClientV1.get.register
def get_node_by_label_v1(client: ClientV1, grai_type: NodeLabels) -> List[NodeV1]:
    url = client.get_url(grai_type)
    resp = client.get(url).json()
    return [NodeV1.from_spec(obj) for obj in resp]


@ClientV1.get.register
def get_node_by_id(
    client: ClientV1, grai_type: NodeLabels, node: str
) -> Optional[NodeV1]:
    url = f"{client.node_endpoint}{node}"
    resp = client.get(url).json()
    return NodeV1.from_spec(resp)


@ClientV1.get.register
def get_edge_v1(client: ClientV1, grai_type: EdgeV1) -> Optional[EdgeV1]:
    base_url = client.edge_endpoint
    if grai_type.spec.id is not None:
        url = f"{base_url}{grai_type.spec.id}"
        resp = client.get(url).json()
    else:
        url = f"{base_url}?name={grai_type.spec.name}&namespace={grai_type.spec.namespace}"
        resp = client.get(url).json()
        if len(resp) == 0:
            return None
        resp = resp[0]

    resp["source"] = client.get("node", resp["source"]).spec
    resp["destination"] = client.get("node", resp["destination"]).spec
    return EdgeV1.from_spec(resp)


@ClientV1.get.register
def get_edge_by_label_v1(client: ClientV1, grai_type: EdgeLabels) -> List[EdgeV1]:
    url = client.get_url(grai_type)
    resp = client.get(url).json()

    for r in resp:
        r["source"] = client.get("node", r["source"]).spec
        r["destination"] = client.get("node", r["destination"]).spec

    return [EdgeV1.from_spec(obj) for obj in resp]



from typing import Callable, Dict, List, Literal, Optional, Type, Union
from uuid import UUID

from pydantic import Field, validator
from typing_extensions import Annotated

from grai_client.schemas.node import NodeID
from grai_client.schemas.utilities import BaseSpec, GraiBaseModel, PlaceHolderSchema


class BaseEdge(GraiBaseModel):
    type: Literal["Edge"]


class V1(BaseSpec):
    id: Optional[UUID]
    name: str
    namespace: str = "default"
    display_name: Optional[str]
    data_source: str
    source: NodeID
    destination: NodeID
    is_active: Optional[bool] = True
    metadata: Optional[Dict] = {}

    def __hash__(self):
        return hash(hash(self.name) + hash(self.namespace))

    def __str__(self):
        return f"Edge[Node({self.source}) -> Node({self.destination})]"


class V2(PlaceHolderSchema, V1):
    """Placeholder for future use."""

    pass


class EdgeV1(BaseEdge):
    version: Literal["v1"]
    spec: V1

    @classmethod
    def from_spec(cls, spec_dict: Dict) -> "EdgeV1":
        args = {
            "version": "v1",
            "type": "Edge",
            "spec": spec_dict,
        }
        return cls(**args)


class EdgeV2(BaseEdge):
    version: Literal["v2"]
    spec: V2

    def from_spec(self, spec_dict: Dict) -> "EdgeV2":
        raise NotImplementedError()


EdgeLabels = Literal["edge", "edges", "Edge", "Edges"]
EdgeTypes = Union[EdgeV1, EdgeV2]
Edge = Annotated[EdgeTypes, Field(discriminator="version")]



import requests

from grai_client.endpoints.client import BaseClient
from grai_client.endpoints.utilities import GraiEncoder, response_status_check


class ClientV1(BaseClient):
    id = "v1"
    base = "/api/v1/"
    _node_endpoint = "lineage/nodes/"
    _edge_endpoint = "lineage/edges/"
    _is_authenticated = "auth/is-authenticated/"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.api = f"{self.url}{self.base}"
        self.node_endpoint = f"{self.api}{self._node_endpoint}"
        self.edge_endpoint = f"{self.api}{self._edge_endpoint}"
        self.is_authenticated_endpoint = f"{self.api}{self._is_authenticated}"

    def check_authentication(self) -> requests.Response:
        result = requests.get(self.is_authenticated_endpoint, headers=self.auth_headers)
        return result



import json
import sys
from functools import wraps
from typing import Any, Callable, Dict, List, Type, TypeVar
from uuid import UUID

from pydantic import BaseModel
from requests import RequestException, Response

from grai_client.schemas.utilities import GraiBaseModel

if sys.version_info < (3, 10):
    from typing_extensions import ParamSpec
else:
    from typing import ParamSpec

P = ParamSpec("P")
T = TypeVar("T")


def response_status_check(resp: Response) -> Response:
    if resp.status_code in {200, 201}:
        return resp
    elif resp.status_code == 204:
        return resp
    elif resp.status_code == 400:
        message = f"400 Bad request: {str(resp.content)} "
    elif resp.status_code in {401, 403}:
        message = f"Failed to Authenticate with code: {resp.status_code}"
    elif resp.status_code == 404:
        message = f"Error: {resp.status_code}. {resp.reason}"
    elif resp.status_code == 405:
        message = f"{resp.status_code} Operation not permitted: {resp.reason}"
    elif resp.status_code == 415:
        message = f"Error: {resp.status_code}. {resp.reason}"
    elif resp.status_code == 500:
        message = (
            f"Error: {resp.status_code}. {resp.reason} "
            "If you think this should not be the case it might be a bug, you can "
            "submit a bug report at https://github.com/grai-io/grai-core/issues"
        )
    else:
        message = f"No handling for error code {resp.status_code}: {resp.reason}"

    raise RequestException(message)


class GraiEncoder(json.JSONEncoder):
    def default(self, obj: Any) -> Any:
        if isinstance(obj, UUID):
            return str(obj)
        elif isinstance(obj, (GraiBaseModel, BaseModel)):
            return obj.dict()
        return json.JSONEncoder.default(self, obj)



from typing import TypeVar

from grai_client.endpoints.v1.client import ClientV1
from grai_client.schemas import edge, node
from grai_client.schemas.utilities import merge_models

T = TypeVar("T", node.NodeV1, edge.EdgeV1)


@ClientV1.patch.register
def patch_obj_v1(client: ClientV1, grai_type: node.NodeV1) -> node.NodeV1:
    if grai_type.spec.id is None:
        current = client.get(grai_type)
        grai_type.spec.id = current.spec.id

    base_url = client.get_url(grai_type)
    url = f"{base_url}{grai_type.spec.id}/"
    response = client.patch(url, grai_type.spec.dict()).json()
    if response is None:
        return None
    return node.NodeV1.from_spec(response)


@ClientV1.patch.register
def patch_obj_v1(client: ClientV1, grai_type: edge.EdgeV1) -> edge.EdgeV1:
    if grai_type.spec.id is None:
        current = client.get(grai_type)
        grai_type.spec.id = current.spec.id

    base_url = client.get_url(grai_type)
    url = f"{base_url}{grai_type.spec.id}/"

    source = grai_type.spec.source
    destination = grai_type.spec.destination
    if source.id is None:
        source = client.get(source)
    if destination.id is None:
        destination = client.get(destination)

    payload = grai_type.spec.dict()
    payload["source"] = source.id
    payload["destination"] = destination.id
    response = client.patch(url, payload).json()
    if response is not None:
        response["source"] = source
        response["destination"] = destination

    return edge.EdgeV1.from_spec(response)



import os


def get_test_client():
    from grai_client.endpoints.v1.client import ClientV1

    host = os.environ.get("GRAI_HOST", "localhost")
    port = os.environ.get("GRAI_PORT", "8000")
    username = os.environ.get("GRAI_USERNAME", "null@grai.io")
    password = os.environ.get("GRAI_PASSWORD", "super_secret")

    client = ClientV1(host, port)
    client.set_authentication_headers(username=username, password=password)
    return client



from typing import Dict, List, Optional, TypeVar

from grai_client.endpoints.client import BaseClient
from grai_client.schemas.edge import Edge
from grai_client.schemas.node import Node
from grai_client.schemas.schema import GraiType, Schema
from grai_client.schemas.utilities import merge_models

T = TypeVar("T", Node, Edge)


def deactivate(items: List[T]) -> List[T]:
    updated = [item.update({"spec": {"is_active": False}}) for item in items]
    return updated


def update(client: BaseClient, items: List[T], active_items: Optional[List[T]] = None):
    if not items:
        return

    if active_items is None:
        active_items: List[T] = client.get(items[0].type)
        if active_items is None:
            active_items = []

    current_item_map = {hash(item.spec): item for item in active_items}
    item_map: Dict[int, T] = {hash(item.spec): item for item in items}

    new_item_keys = new_items - active_items
    deactivated_item_keys = current_item_map.keys() - item_map.keys()
    updated_item_keys = item_map.keys() - new_item_keys

    new_items: List[T] = [item_map[k] for k in new_item_keys]
    updated_items = [
        merge_models(item_map[k], current_item_map[k])
        for k in updated_item_keys
        if item_map[k] != current_item_map[k]
    ]
    client.patch(deactivate(active_items - new_items))
    client.patch(updated_items)
    [new_items[k] for k in new_items.keys() - active_items.keys()]
    client.post(new_items - active_items)



import json
from typing import Any, Dict, List, Type

import requests

from grai_client.endpoints.utilities import GraiEncoder, response_status_check
from grai_client.endpoints.v1.client import ClientV1
from grai_client.schemas.edge import EdgeV1
from grai_client.schemas.node import NodeV1


@ClientV1.delete.register
def delete_node_v1(client: ClientV1, grai_type: NodeV1):
    if grai_type.spec.id is None:
        grai_type = client.get(grai_type)
    url = f"{client.node_endpoint}{grai_type.spec.id}/"
    client.delete(url)


@ClientV1.delete.register
def delete_edge_v1(client: ClientV1, grai_type: EdgeV1):
    if grai_type.spec.id is None:
        grai_type = client.get(grai_type)
        if grai_type is None:
            return
    url = f"{client.edge_endpoint}{grai_type.spec.id}/"
    client.delete(url)



from pathlib import Path
from typing import Any, Dict, Iterable, Type, Union

import yaml
from pydantic import Field
from typing_extensions import Annotated

from grai_client.schemas.edge import Edge
from grai_client.schemas.node import Node
from grai_client.schemas.utilities import GraiBaseModel

GraiType = Annotated[Union[Node, Edge], Field(discriminator="type")]


class Schema(GraiBaseModel):
    entity: GraiType

    @classmethod
    def to_model(cls, item: Dict, version: str, typing_type: str) -> GraiType:
        result = {
            "type": typing_type,
            "version": version,
            "spec": item,
        }
        return cls(entity=result).entity


def validate_file(file: Union[str, Path]) -> Iterable[GraiType]:
    with open(file) as f:
        for item in yaml.safe_load_all(f):
            yield Schema(entity=item).entity



import abc
import json

import requests

json_headers = {"accept": "application/json", "Content-Type": "application/json"}


class AuthHeader(abc.ABC):
    @property
    def headers(self):
        pass


class UserTokenHeader(AuthHeader):
    def __init__(self, token: str):
        self.token = token

    @property
    def headers(self):
        return {"Authorization": f"Token {self.token}"}


class APIKeyHeader(AuthHeader):
    def __init__(self, api_key: str):
        self.api_key = api_key

    @property
    def headers(self):
        return {"Authorization": f"Api-Key {self.api_key}"}


class UserNameHeader(UserTokenHeader):
    def __init__(self, username, password, base_url="http://localhost:8000"):
        self.username = username
        self.password = password
        self.base_url = base_url
        super().__init__(token=self.get_token())

    def get_token(self) -> str:
        params = {
            "username": self.username,
            "password": self.password,
        }
        url = f"{self.base_url}/api/v1/auth/api-token/"
        token = requests.post(url, data=json.dumps(params), headers=json_headers)
        if token.status_code != 200:
            raise Exception("Failed to get user token from server")
        return token.json()["token"]
